#include <stdio.h>
#include <stdlib.h>
#include <WinSock2.h>
//#include <string>

#pragma comment(lib,"ws2_32.lib") //Winsock Library

//folyamat
//csatlakozs (192.168.1.1) -> TCP kapcsolat (port 21718)
//parancs kldse 0x03 + 32bit hossz
//binris kldse, max 800byte / csomag
//adat kiegsztse 16bte hosszra
//adatcsomag
//00 32bit offset, 32bit hossz

char packet[809];
char* bin;
WSADATA wsa;

char* read_file(const char* filename, long* length)
{
	FILE *fp = fopen(filename, "rb");
	char* buf = 0;

	if (fp != NULL)
	{
		/* Go to the end of the file. */
		if (fseek(fp, 0L, SEEK_END) == 0)
		{
			/* Get the size of the file. */
			*length = ftell(fp);
			if (*length == -1) { /* Error */ };

			/* Allocate our buffer to that size. */
			buf = (char*)malloc(sizeof(char) * (*length));
			if (!buf) printf("malloc error, no mem space...");

			/* Go back to the start of the file. */
			if (fseek(fp, 0L, SEEK_SET) != 0) { /* Error */ };

			/* Read the entire file into memory. */
			long fsize;
			fsize = fread(buf, sizeof(char), *length, fp);
			if (ferror(fp) != 0) fputs("Error reading file", stderr);
		}

		fclose(fp);
	}

	return buf;
}

void init_wsa()
{
	printf("\nInitialising Winsock...\n");
	if (WSAStartup(MAKEWORD(2, 2), &wsa) != 0)
	{
		printf("Failed. Error Code : %d", WSAGetLastError());
		return 1;
	}

	printf("Initialised\n");
}

int main(int argc, char** argv)
{
	int errcode = 0;
	char * binary = 0;
	long binlength = 0;

	//beolvasa fjlbl
	binary = read_file("firmware.bin", &binlength);
	if (binary) printf("Firmware read successfully\nfilesize: %d\npackets needed: %d\n", binlength, binlength/800 + 1);
	else 
	{	
		printf("Cannot read firmware file\n");
		printf("Binary did not allocated memory.\nExiting.\n");
		return 4;
	}
	
	//szerverhez csatlakozs
	init_wsa();

	//Create a socket
	SOCKET s;
	struct sockaddr_in server;
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
	{
		printf("Could not create socket : %d", WSAGetLastError());
	}
	printf("Socket created.\n");


	server.sin_addr.s_addr = inet_addr("192.168.1.1");
	server.sin_family = AF_INET;
	server.sin_port = htons(21718);

	//Connect to remote server
	printf("Connecting to 192.168.1.1:21718\n");
	if (connect(s, (struct sockaddr *)&server, sizeof(server)) < 0)
	{
		puts("connect error");
		return 1;
	}
	printf("Connected\n");//wait for response
	errcode = recv(s, packet, 800, 0);

	if (errcode < 0)
	{
		closesocket(s);
		puts("server slosed the connection\n");
		return 1;
	}
	
	for (int i = 0; i < errcode; i++) putc(packet[i], stdout);
	

	//send init packet
	packet[0] = 0x03; //fwupdate mode
	packet[1] = (binlength >> 24) & 0xff;
	packet[2] = (binlength >> 16) & 0xff;
	packet[3] = (binlength >> 8) & 0xff;
	packet[4] = (binlength >> 0) & 0xff;
	
	if (send(s, packet, 5, 0) < 0)
	{
		puts("Send failed");
		return 1;
	}
	puts("Update command sent\n");

	//wait for response
	errcode = recv(s, packet, 800, 0);

	if (errcode < 0)
	{
		closesocket(s);
		puts("server slosed the connection\n");
		return 1;
	}

	else if (packet[0] == 'O')
	{
		puts("server ready to receive\n");
	}
	else
	{
		puts("error, cannot send fw\n");
		closesocket(s);
		return 2;
	}

	//send data
	int n_pack = binlength / 800;
	if (binlength % 800) n_pack++;

	unsigned long offset;
	unsigned long datalen;
	int padding;
	for (int i = 0; i < n_pack; i++)
	{
		offset = 800 * i;
		datalen = binlength - offset;
		if (datalen > 800) datalen = 800;

		//csomag sszelltsa
		//fejlc
		packet[0] = 0;
		packet[1] = (offset >> 24) & 0xff;
		packet[2] = (offset >> 16) & 0xff;
		packet[3] = (offset >> 8) & 0xff;
		packet[4] = (offset >> 0) & 0xff;
		packet[5] = (datalen >> 24) & 0xff;
		packet[6] = (datalen >> 16) & 0xff;
		packet[7] = (datalen >> 8) & 0xff;
		packet[8] = (datalen >> 0) & 0xff;

		for (int j = 0; j < datalen; j++) packet[9 + j] = binary[offset + j];

		//padding
		padding = datalen % 16;
		for (int j = 0; j < padding; j++) packet[9 + datalen + j] = 0;
		datalen += padding;

		//csomag elkldse
		if (send(s, packet, datalen + 9, 0) < 0)
		{
			puts("Send failed");
			return 1;
		}

		//vrakozs az OK-ra
		//wait for response
		errcode = recv(s, packet, 800, 0);

		if (errcode < 0)
		{
			closesocket(s);
			puts("server slosed the connection\n");
			return 1;
		}

		if (packet[0] == 'O')
		{
			printf("\r%d/%d bytes sent [%2.1lf%%]", offset + datalen, binlength, 100.0*(float)(offset + datalen) / (float)binlength);
		}
		else
		{
			printf("\nPacket send error...\nEXITING\n");
			closesocket(s);
			return 3;
		}
	}

	printf("\n\nFIRMWARE UPDATE DONE\n");
	closesocket(s);
	
	free(binary);
	
	return 0;
}